@@ -139,14 +139,25 @@ PositionDodgeAndNudge <-
 
                    setup_params = function(self, data) {
                      c(
-                       list(nudge_x = self$x, nudge_y = self$y,
-                            .fun_x = self$.fun_x, .fun_y = self$.fun_y,
+                       list(nudge_x = self$x,
+                            nudge_y = self$y,
+                            .fun_x = self$.fun_x,
+                            .fun_y = self$.fun_y,
+                            x.reorder = !is.null(self$x) && length(self$x) > 1 && length(self$x) < nrow(data),
+                            y.reorder = !is.null(self$y) && length(self$y) > 1 && length(self$y) < nrow(data),
                             kept.origin = self$kept.origin),
                        ggplot2::ggproto_parent(ggplot2::PositionDodge, self)$setup_params(data)
                      )
                    },
 
                    compute_layer = function(self, data, params, layout) {
+
+                     if (length(params$nudge_x) > nrow(data)) {
+                       warning("Argument 'x' longer than data: some values dropped!")
+                     }
+                     if (length(params$nudge_y) > nrow(data)) {
+                       warning("Argument 'y' longer than data: some values dropped!")
+                     }
                      x_orig <- data$x
                      y_orig <- data$y
                      # operate on the dodged positions
@@ -154,6 +165,18 @@ PositionDodgeAndNudge <-
 
                      x_dodged <- data$x
                      y_dodged <- data$y
+
+                     if (params$x.reorder) {
+                       params$nudge_x <- rep_len(params$nudge_x, nrow(data))[order(order(data$x))]
+                     } else {
+                       params$nudge_x <- rep_len(params$nudge_x, nrow(data))
+                     }
+                     if (params$y.reorder) {
+                       params$nudge_y <- rep_len(params$nudge_y, nrow(data))[order(order(data$y))]
+                     } else {
+                       params$nudge_y <- rep_len(params$nudge_y, nrow(data))
+                     }
+
                      # transform only the dimensions for which non-zero nudging is requested
                      if (any(params$nudge_x != 0)) {
                        if (any(params$nudge_y != 0)) {
@@ -192,13 +215,14 @@ PositionDodgeAndNudge <-
 #' @export
 #'
 position_dodge_keep <- function(width = 1,
-                                preserve = c("total", "single")) {
+                                preserve = c("total", "single"),
+                                kept.origin = "original") {
   position_dodgenudge(width = width,
                       preserve = preserve,
                       x = 0,
                       y = 0,
                       direction = "none",
-                      kept.origin = "original")
+                      kept.origin = kept.origin)
 }
 
 #' @rdname position_dodgenudge
@@ -206,11 +230,12 @@ position_dodge_keep <- function(width = 1,
 #' @export
 #'
 position_dodge2_keep <- function(width = 1,
-                                 preserve = c("total", "single")) {
+                                 preserve = c("total", "single"),
+                                 kept.origin = "original") {
   position_dodge2nudge(width = width,
                        preserve = preserve,
                        x = 0,
                        y = 0,
                        direction = "none",
-                       kept.origin = "original")
+                       kept.origin = kept.origin)
 }
